import { isConditionalExpression, isJSXElement } from "./utils-CInUWtxo.js";
import { transformJSX } from "./transform-B3iRdXnn.js";
import { parse } from "@babel/parser";
import _SyntaxJSX from "@babel/plugin-syntax-jsx";

//#region src/index.ts
const SyntaxJSX = _SyntaxJSX.default || _SyntaxJSX;
var src_default = () => {
	return {
		name: "Vue JSX Vapor",
		inherits: SyntaxJSX,
		visitor: {
			JSXElement: transformJSX,
			JSXFragment: transformJSX,
			Program: {
				enter: (path, state) => {
					state.importSet = new Set();
					state.delegateEventSet = new Set();
					state.preambleMap = new Map();
					state.preambleIndex = 0;
					state.roots = [];
					const collectRoot = (path$1) => {
						if (!isJSXElement(path$1.parent) && !isConditionalExpression(path$1.parentPath)) state.roots.push({
							node: path$1.node,
							source: path$1.getSource(),
							inVaporComponent: !state.opts.interop ? true : path$1.findParent(({ node }) => node.type === "CallExpression" && node.callee.type === "Identifier" && ["defineVaporComponent", "defineComponent"].includes(node.callee.name))?.node.callee.name === "defineVaporComponent"
						});
					};
					path.traverse({
						JSXElement: collectRoot,
						JSXFragment: collectRoot
					});
				},
				exit: (path, state) => {
					const { delegateEventSet, importSet, preambleMap } = state;
					const statements = [];
					if (delegateEventSet.size) statements.unshift(`_delegateEvents(${Array.from(delegateEventSet).join(", ")});`);
					if (preambleMap.size) {
						let preambleResult = "";
						for (const [value, key] of preambleMap) preambleResult += `const ${key} = ${value}\n`;
						statements.unshift(preambleResult);
					}
					const helpers = ["setNodes", "createNodes"].filter((helper) => {
						const result = importSet.has(helper);
						result && importSet.delete(helper);
						return result;
					});
					if (helpers.length) statements.unshift(`import { ${helpers.map((i) => `${i} as _${i}`).join(", ")} } from 'vue-jsx-vapor/runtime';\n`);
					if (importSet.size) {
						const importResult = Array.from(importSet).map((i) => `${i} as _${i}`).join(", ");
						statements.unshift(`import { ${importResult} } from 'vue';\n`);
					}
					path.node.body.unshift(...parse(statements.join("\n"), { sourceType: "module" }).program.body);
				}
			}
		}
	};
};

//#endregion
export { src_default as default };